from django.apps import apps as django_apps
from django.test import TestCase
from edc_appointment.models import Appointment
from edc_visit_tracking.constants import SCHEDULED

from edc_action_item.models import ActionItem
from edc_action_item.utils import (
    get_parent_reference_obj,
    get_reference_obj,
    get_related_reference_obj,
)

from ..action_items import CrfOneAction, register_actions
from ..models import CrfOne, CrfTwo, FormOne, FormTwo
from ..test_case_mixin import TestCaseMixin


class TestHelpers(TestCaseMixin, TestCase):
    def setUp(self):
        register_actions()
        self.subject_identifier = self.fake_enroll()
        self.form_one = FormOne.objects.create(subject_identifier=self.subject_identifier)
        self.action_item = ActionItem.objects.get(
            action_identifier=self.form_one.action_identifier
        )

    def test_new_action(self):
        CrfOneAction(subject_identifier=self.subject_identifier)
        self.assertIsNone(get_reference_obj(None))
        self.assertIsNone(get_parent_reference_obj(None))
        self.assertIsNone(get_related_reference_obj(None))

    def test_create_parent_reference_model_instance_then_delete(self):
        form_two = FormTwo.objects.create(
            form_one=self.form_one, subject_identifier=self.subject_identifier
        )
        action_item = ActionItem.objects.get(action_identifier=form_two.action_identifier)
        self.assertEqual(get_reference_obj(action_item), form_two)
        form_two.delete()
        action_item = ActionItem.objects.get(action_identifier=form_two.action_identifier)
        self.assertIsNone(get_reference_obj(action_item))

    def test_create_parent_reference_model_instance(self):
        form_two = FormTwo.objects.create(
            form_one=self.form_one, subject_identifier=self.subject_identifier
        )
        action_item = ActionItem.objects.get(action_identifier=form_two.action_identifier)
        self.assertEqual(get_reference_obj(action_item), form_two)
        self.assertEqual(get_parent_reference_obj(action_item), self.form_one)
        self.assertEqual(get_related_reference_obj(action_item), self.form_one)

    def test_create_next_parent_reference_model_instance(self):
        first_form_two = FormTwo.objects.create(
            form_one=self.form_one, subject_identifier=self.subject_identifier
        )
        second_form_two = FormTwo.objects.create(
            form_one=self.form_one, subject_identifier=self.subject_identifier
        )
        action_item = ActionItem.objects.get(
            action_identifier=second_form_two.action_identifier
        )
        self.assertEqual(get_reference_obj(action_item), second_form_two)
        self.assertEqual(get_parent_reference_obj(action_item), first_form_two)
        self.assertEqual(get_related_reference_obj(action_item), self.form_one)

    def test_reference_as_crf(self):
        self.enroll()
        appointment = Appointment.objects.all().order_by("timepoint", "visit_code_sequence")[0]
        subject_visit = django_apps.get_model(
            "edc_visit_tracking.subjectvisit"
        ).objects.create(appointment=appointment, reason=SCHEDULED)
        crf_one = CrfOne.objects.create(subject_visit=subject_visit)
        action_item = ActionItem.objects.get(action_identifier=crf_one.action_identifier)
        self.assertEqual(get_reference_obj(action_item), crf_one)
        self.assertIsNone(get_parent_reference_obj(action_item))
        self.assertIsNone(get_related_reference_obj(action_item))

    def test_reference_as_crf_create_next_model_instance(self):
        self.enroll()
        appointment = Appointment.objects.all().order_by("timepoint", "visit_code_sequence")[0]
        subject_visit = django_apps.get_model(
            "edc_visit_tracking.subjectvisit"
        ).objects.create(appointment=appointment, reason=SCHEDULED)
        crf_one = CrfOne.objects.create(subject_visit=subject_visit)
        crf_two = CrfTwo.objects.create(subject_visit=subject_visit)
        action_item = ActionItem.objects.get(action_identifier=crf_two.action_identifier)
        self.assertEqual(get_reference_obj(action_item), crf_two)
        self.assertEqual(get_parent_reference_obj(action_item), crf_one)
        self.assertIsNone(get_related_reference_obj(action_item))



from dateutil.relativedelta import relativedelta
from django.test import TestCase
from edc_consent.site_consents import site_consents
from edc_facility.import_holidays import import_holidays
from edc_registration.models import RegisteredSubject
from edc_utils import get_utcnow
from edc_visit_schedule.site_visit_schedules import site_visit_schedules

from .visit_schedule import visit_schedule


class TestCaseMixin(TestCase):
    @classmethod
    def setUpTestData(cls):
        import_holidays()

    @staticmethod
    def enroll(subject_identifier=None):
        subject_identifier = subject_identifier or "1111111"

        site_visit_schedules._registry = {}
        site_visit_schedules.loaded = False
        site_visit_schedules.register(visit_schedule)

        consent_datetime = get_utcnow()
        cdef = site_consents.get_consent_definition(report_datetime=consent_datetime)
        subject_consent = cdef.model_cls.objects.create(
            subject_identifier=subject_identifier,
            consent_datetime=consent_datetime,
            dob=get_utcnow() - relativedelta(years=25),
        )
        RegisteredSubject.objects.create(subject_identifier=subject_identifier)
        _, schedule = site_visit_schedules.get_by_onschedule_model(
            "edc_visit_schedule.onschedule"
        )
        schedule.put_on_schedule(
            subject_consent.subject_identifier, subject_consent.consent_datetime
        )
        return subject_identifier

    @staticmethod
    def fake_enroll():
        subject_identifier = "2222222"
        RegisteredSubject.objects.create(subject_identifier=subject_identifier)
        return subject_identifier



from django.test import RequestFactory, TestCase
from django.views.generic.base import ContextMixin, View
from edc_sites.utils import get_site_model_cls
from edc_test_utils.get_user_for_tests import get_user_for_tests

from edc_action_item.models import ActionItem, ActionType
from edc_action_item.view_mixins import ActionItemViewMixin

from ..test_case_mixin import TestCaseMixin


class MyActionItemViewMixin(ActionItemViewMixin, ContextMixin, View):
    pass


class TestAction(TestCaseMixin, TestCase):
    def setUp(self):
        self.subject_identifier = self.fake_enroll()
        self.user = get_user_for_tests()
        self.site = get_site_model_cls().objects.get_current()

    def test_view_context(self):
        req = RequestFactory().get("/")
        req.user = self.user
        req.site = self.site
        view = MyActionItemViewMixin()
        view.request = req
        view.kwargs = dict(subject_identifier=self.subject_identifier)
        context = view.get_context_data()
        self.assertEqual(context.get("open_action_items").count(), 0)

        for action_type in ActionType.objects.all():
            ActionItem.objects.create(
                subject_identifier=self.subject_identifier, action_type=action_type
            )

        view = MyActionItemViewMixin()
        view.request = req
        view.kwargs = dict(subject_identifier=self.subject_identifier)
        context = view.get_context_data()
        self.assertEqual(
            len(context.get("open_action_items")), ActionItem.objects.all().count()
        )
